package Gear::Remotes;
use strict;
use warnings;
use Gear::Rules;
use Gear::Remotes::File;

our $VERSION = 0.021;
our $verbose=0;

sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
	@_,
    };
    bless($self,$class);
    my $topdir=Gear::Git::Helper::get_git_toplevel();

    die "fatal: $topdir - not a gear repository.\n" unless &check_if_gear_dir($topdir);

    my $gear_remotes_file=&get_gear_remotes_file($topdir);
    my $git_config_file=$topdir.'.git/config';
    my $gear_remotes_obj=Gear::Remotes::File->new(-file => $gear_remotes_file);
    my $git_remotes_obj=Gear::Remotes::File->new(-file => $git_config_file);

    my %in_git=map{$_=>1} $git_remotes_obj->list();
    my (@entries_in_git,@entries_out_git);
    my (%svn_remotes,%git_remotes);
    foreach my $entry ($gear_remotes_obj->list()) {
	if ($in_git{$entry}) {
	    push @entries_in_git, $entry;
	} else {
	    push @entries_out_git, $entry;
	}
	if ($entry=~/^(svn-)?remote\.([^\.]+)\.url=(.*)/) {
	    my $svn=$1;
	    my $remotesname=$2;
	    my $url=$3;
	    $url=~s,^[^:]+://,,;
	    if ($url=~m,git(?:ery)?(\.alt(?:linux\.(?:org|ru)?)?)[:/],) {
		die "$gear_remotes_file: invalid remotes $entry: gitery.alt as upstream remotes makes no sense.\n";
	    } else {
		if ($svn) {
		    $svn_remotes{$remotesname}=1;
		} else {
		    $git_remotes{$remotesname}=1;
		}
	    }
	}
    }
    
    if (@entries_out_git and not @entries_in_git) {
	if ($self->{-autorestore}) {
	    `cat $gear_remotes_file >> $git_config_file`;
	} else {
	    die "Fatal: remotes listed in $gear_remotes_file are not present in $git_config_file. Run gear-remotes-restore first.\n";
	}
    } elsif (@entries_out_git and @entries_in_git) {
	print STDERR "entries in $gear_remotes_file and in $git_config_file\n";
	map {print STDERR "\t$_\n"} @entries_in_git;
	print STDERR "entries in $gear_remotes_file not in $git_config_file\n";
	map {print STDERR "\t$_\n"} @entries_out_git;
	die "Fatal: remotes listed in $gear_remotes_file conflicts with $git_config_file. Resolve conflict first.\n";
    }
    $self->{-git_remotes}=[sort keys (%git_remotes)];
    $self->{-svn_remotes}=[sort keys (%svn_remotes)];
    return $self;
}

sub check_if_gear_dir {
    my $topdir=shift;
    my $test=-d $topdir.'.git' and (-d $topdir.'.gear' or -f $topdir.'.gear-rules');
    return $test;
}

sub get_gear_remotes_file {
    my $topdir=shift;
    my $gear_remotes_file=$topdir.'.gear/upstream/remotes';
    $gear_remotes_file=$topdir.'.gear/upstream/remotes.'.$ENV{'USER'} if -e $topdir.'.gear/upstream/remotes.'.$ENV{'USER'};
    my $suffix=$ENV{'GEAR_UPSTREAM_REMOTES_SUFFIX'};
    $gear_remotes_file=$topdir.'.gear/upstream/remotes'.$suffix if defined($suffix) and -e $topdir.'.gear/upstream/remotes'.$suffix;
    return $gear_remotes_file;
}

sub fetch {
    my ($self)=@_;
    foreach my $remote (@{$self->{-svn_remotes}}) {
	print "git svn fetch $remote\n";
	system (qw/git svn fetch/,$remote)==0 or die "ERROR: git-svn fetch $remote failed\n";
    }

    foreach my $remote (@{$self->{-git_remotes}}) {
	print "git fetch $remote\n";
	system (qw/git fetch --recurse-submodules/,$remote)==0 or die "ERROR: git fetch $remote failed\n";
    }
}

1;

__END__
