package Gear::Remotes::SpecComment;

use strict;
use warnings;
use Gear::Rules;
use RPM::Source::Editor 0.915;

sub get_spec_comment {
    my $rules=Gear::Rules->new();
    my $specfile=$rules->get_spec;
    unless (defined $specfile and -e $specfile) {
	die "Oops! internal: bad specfile $specfile";
    }

    my $spec=RPM::Source::Editor->new(
	SPECFILE=>$specfile,
	);

    my ($sourcegit,$comment);
    my $priority=0;
    my $_set_scp = sub {
	($comment,$sourcegit,$priority)=@_ if $priority < $_[2];
    };

    $spec->main_section->visit_body(
	sub {
	    if (/^(#\s*Source(?:\d+)?-git:)\s+(\S+?)\s*$/) {
		&$_set_scp($1,$2,5);
	    } elsif (m!^(#\s*(?:VCS URL|VCS|Vcs-git|GIT|Upstream|upstream|Cloned\s+from(?:\s+git)?):)\s+((?:git://|https?://github)\S+?)\s*$!) {
		&$_set_scp($1,$2,4);
	    } elsif (m!^(#\s*git(?:\s+clone)?)\s+((?:git://|https?://github)\S+?)\s*$!) {
		&$_set_scp($1,$2,3);
	    } elsif (m!^(#\s*[Cc]loned\s+from(?:\s+git)?)\s+((?:git://|https?://github)\S+?)\s*$!) {
		&$_set_scp($1,$2,3);
	    } elsif (m!^(#)\s+((?:git://|https?://github)\S+?)\s*$!) {
		&$_set_scp($1,$2,2);
	    } elsif (m!^(#\s*(?:Url|URL):)\s+((?:git://|https?://github)\S+?)\s*$!) {
		&$_set_scp($1,$2,2);
	    } elsif (m!^(#\s*(?:git)?)\s+(https?://git\S+?)\s*$!) {
		&$_set_scp($1,$2,2);
	    } elsif (m!^(#\s*git:?)\s+(\S+?)\s*$!) {
		&$_set_scp($1,$2,1);
	    }
	});
    return ($sourcegit,$comment);
}

1;
